#include <QtGui>
#include <string>
#include <iostream>
#include "highlighter.h"
using namespace std;

Highlighter::Highlighter(QTextDocument *parent) : QSyntaxHighlighter(parent){
    HighlightingRule rule;

	/*
	* VALUES
	*/
    valuesFormat.setForeground(Qt::red);
    valuesFormat.setFontWeight(QFont::Bold);
    QStringList valuePatterns;

	valuePatterns << "\\babove\\b" << "\\babsolute\\b" << "\\balways\\b" << "\\barmenian\\b" << "\\baural\\b" << "\\bauto\\b" <<
             "\\bavoid\\b" << "\\bbaseline\\b" << "\\bbehind\\b" << "\\bbelow\\b" << "\\bbidi-override\\b" << "\\bblink\\b" <<
             "\\bblock\\b" << "\\bbold\\b" << "\\bbolder\\b" << "\\bboth\\b" << "\\bcapitalize\\b" << "\\bcenter-left\\b" <<
             "\\bcenter-right\\b" << "\\bcenter\\b" << "\\bcircle\\b" << "\\bcjk-ideographic\\b" <<
             "\\bclose-quote\\b" << "\\bcollapse\\b" << "\\bcondensed\\b" << "\\bcontinuous\\b" << "\\bcrop\\b" <<
             "\\bcrosshair\\b" << "\\bcross\\b" << "\\bcursive\\b" << "\\bdashed\\b" << "\\bdecimal-leading-zero\\b" <<
             "\\bdecimal\\b" << "\\bdefault\\b" << "\\bdigits\\b" << "\\bdisc\\b" << "\\bdotted\\b" << "\\bdouble\\b" <<
             "\\be-resize\\b" << "\\bembed\\b" << "\\bextra-condensed\\b" << "\\bextra-expanded\\b" <<
             "\\bexpanded\\b" << "\\bfantasy\\b" << "\\bfar-left\\b" << "\\bfar-right\\b" << "\\bfaster\\b" << "\\bfast\\b" <<
             "\\bfixed\\b" << "\\bfuchsia\\b" << "\\bgeorgian\\b" << "\\bgray\\b" << "\\bgreen\\b" << "\\bgroove\\b" <<
             "\\bhebrew\\b" << "\\bhelp\\b" << "\\bhidden\\b" << "\\bhide\\b" << "\\bhigher\\b" << "\\bhigh\\b" <<
             "\\bhiragana-iroha\\b" << "\\bhiragana\\b" << "\\bicon\\b" << "\\binherit\\b" << "\\binline-table\\b" <<
             "\\binline\\b" << "\\binset\\b" << "\\binside\\b" << "\\binvert\\b" << "\\bitalic\\b" << "\\bjustify\\b" <<
             "\\bkatakana-iroha\\b" << "\\bkatakana\\b" << "\\blandscape\\b" << "\\blarger\\b" << "\\blarge\\b" <<
             "\\bleft-side\\b" << "\\bleftwards\\b" << "\\blevel\\b" << "\\blighter\\b" << "\\blime\\b" <<
             "\\bline-through\\b" << "\\blist-item\\b" << "\\bloud\\b" << "\\blower-alpha\\b" << "\\blower-greek\\b" <<
             "\\blower-roman\\b" << "\\blowercase\\b" << "\\bltr\\b" << "\\blower\\b" << "\\blow\\b" << "\\bmaroon\\b" <<
             "\\bmedium\\b" << "\\bmessage-box\\b" << "\\bmiddle\\b" << "\\bmix\\b" << "\\bmonospace\\b" << "\\bn-resize\\b" <<
             "\\bnarrower\\b" << "\\bnavy\\b" << "\\bno-resize\\b" << "\\bno-close-quote\\b" <<
             "\\bno-open-quote\\b" << "\\bno-repeat\\b" << "\\bnone\\b" << "\\bnormal\\b" << "\\bnowrap\\b" <<
             "\\bnw-resize\\b" << "\\boblique\\b" << "\\bolive\\b" << "\\bonce\\b" << "\\bopen-quote\\b" << "\\boutset\\b" <<
             "\\boutside\\b" << "\\boverline\\b" << "\\bpointer\\b" << "\\bportrait\\b" << "\\bpurple\\b" << "\\bpx\\b" <<
             "\\bred\\b" << "\\brelative\\b" << "\\brepeat-x\\b" << "\\brepeat-y\\b" << "\\brepeat\\b" << "\\brgb\\b" <<
             "\\bridge\\b" << "\\bright-side\\b" << "\\brightwards\\b" << "\\bs-resize\\b" << "\\bsans-serif\\b" <<
             "\\bscroll\\b" << "\\bse-resize\\b" << "\\bsemi-condensed\\b" << "\\bsemi-expanded\\b" <<
             "\\bseparate\\b" << "\\bserif\\b" << "\\bshow\\b" << "\\bsilent\\b" << "\\bsilver\\b" << "\\bslow\\b" << "\\bslower\\b" <<
             "\\bsmall-caps\\b" << "\\bsmall-caption\\b" << "\\bsmaller\\b" << "\\bsoft\\b" << "\\bsolid\\b" <<
             "\\bspell-out\\b" << "\\bsquare\\b" << "\\bstatic\\b" << "\\bstatus-bar\\b" << "\\bsuper\\b" <<
             "\\bsw-resize\\b" << "\\btable-caption\\b" << "\\btable-cell\\b" << "\\btable-column\\b" <<
             "\\btable-column-group\\b" << "\\btable-footer-group\\b" << "\\btable-header-group\\b" <<
             "\\btable-row\\b" << "\\btable-row-group\\b" << "\\bteal\\b" << "\\btext\\b" << "\\btext-bottom\\b" <<
             "\\btext-top\\b" << "\\bthick\\b" << "\\bthin\\b" << "\\btransparent\\b" << "\\bultra-condensed\\b" <<
             "\\bultra-expanded\\b" << "\\bunderline\\b" << "\\bupper-alpha\\b" << "\\bupper-latin\\b" <<
             "\\bupper-roman\\b" << "\\buppercase\\b" << "\\burl\\b" << "\\bvisible\\b" << "\\bw-resize\\b" << "\\bwait\\b" <<
             "\\bwhite\\b" << "\\bwider\\b" << "\\bx-fast\\b" << "\\bx-high\\b" << "\\bx-large\\b" << "\\bx-loud\\b" <<
             "\\bx-low\\b" << "\\bx-small\\b" << "\\bx-soft\\b" << "\\bxx-large\\b" << "\\bxx-small\\b" << "\\byellow\\b" << "\\byes\\b" << 
             "\\b[0-9]*(px|em)\\b" << "\\b[0-9]*\\b%" <<
             "[^-]\\bleft\\b" << "[^-]\\bright\\b" << "[^-]\\bcenter\\b" << 
             "\\bVerdana\\b" << "\\bArial\\b" << "\\bHelvetica\\b" << "\\bsans-serif\\b" <<
             "#\\b([a-zA-Z0-0]{3}|[a-zA-Z0-0]{6})\\b";

    foreach (QString pattern, valuePatterns) {
        rule.pattern = QRegExp(pattern, Qt::CaseInsensitive);
        rule.format = valuesFormat;
        highlightingRules.append(rule);
    }
	/* END KEYWORDS */

	/*
	* KEYWORDS
	*/
    keywordFormat.setForeground(Qt::darkBlue);
    keywordFormat.setFontWeight(QFont::Bold);
    QStringList keywordPatterns;

	keywordPatterns << "\\baqua\\b" << "\\bazimuth\\b" << "\\bbackground-attachment\\b" << "\\bbackground-color\\b" <<
            "\\bbackground-image\\b" << "\\bbackground-position\\b" << "\\bbackground-repeat\\b" <<
            "\\bbackground\\b" << "\\bblack\\b" << "\\bblue\\b" << "\\bborder-bottom-color\\b" <<
            "\\bborder-bottom-style\\b" << "\\bborder-bottom-width\\b" << "\\bborder-left-color\\b" <<
            "\\bborder-left-style\\b" << "\\bborder-left-width\\b" << "\\bborder-right\\b" <<
            "\\bborder-right-color\\b" << "\\bborder-right-style\\b" << "\\bborder-right-width\\b" <<
            "\\bborder-top-color\\b" << "\\bborder-top-style\\b" <<
            "\\bborder-top-width" << "border-bottom\\b" << "\\bborder-collapse\\b" <<
            "\\bborder-left\\b" << "\\bborder-width\\b" << "\\bborder-color\\b" << "\\bborder-spacing\\b" <<
            "\\bborder-style\\b" << "\\bborder-top\\b" << "\\bborder\\b" << "\\bcaption-side\\b" << "\\bclear\\b" <<
            "\\bclip\\b" << "\\bcolor\\b" << "\\bcontent\\b" << "\\bcounter-increment\\b" << "\\bcounter-reset\\b" <<
            "\\bcue-after\\b" << "\\bcue-before\\b" << "\\bcue\\b" << "\\bcursor\\b" << "\\bdirection\\b" <<
            "\\bdisplay\\b" <<
            "\\belevation\\b" << "\\bempty-cells\\b" << "\\bfloat\\b" << "\\bfont-family\\b" << "\\bfont-size\\b" <<
            "\\bfont-size-adjust\\b" << "\\bfont-stretch\\b" << "\\bfont-style\\b" << "\\bfont-variant\\b" <<
            "\\bfont-weight\\b" << "\\bfont\\b" << "\\bline-height\\b" << "\\bletter-spacing\\b" <<
            "\\blist-style\\b" << "\\blist-style-image\\b" << "\\blist-style-position\\b" <<
            "\\blist-style-type\\b" << "\\bmargin-bottom\\b" << "\\bmargin-left\\b" << "\\bmargin-right\\b" <<
            "\\bmargin-top\\b" << "\\bmargin\\b" << "\\bmarker-offset\\b" << "\\bmarks\\b" << "\\bmax-height\\b" <<
            "\\bmax-width\\b" << "\\bmin-height\\b" << "\\bmin-width\\b" << "\\borphans\\b" << "\\boutline\\b" <<
            "\\boutline-color\\b" << "\\boutline-style\\b" << "\\boutline-width\\b" << "\\boverflow\\b" <<
            "\\bpadding-bottom\\b" << "\\bpadding-left\\b" << "\\bpadding-right\\b" << "\\bpadding-top\\b" <<
            "\\bpadding\\b" << "\\bpage\\b" << "\\bpage-break-after\\b" << "\\bpage-break-before\\b" <<
            "\\bpage-break-inside\\b" << "\\bpause-after\\b" << "\\bpause-before\\b" << "\\bpause\\b" <<
            "\\bpitch\\b" << "\\bpitch-range\\b" << "\\bplay-during\\b" << "\\bposition\\b" << "\\bquotes\\b" <<
            "\\brichness\\b" << "\\bright\\b" << "\\bsize\\b" << "\\bspeak-header\\b" << "\\bspeak-numeral\\b" <<
            "\\bspeak-punctuation\\b" << "\\bspeak\\b" << "\\bspeech-rate\\b" << "\\bstress\\b" <<
            "\\btable-layout\\b" << "\\btext-align\\b" << "\\btext-decoration\\b" << "\\btext-indent\\b" <<
            "\\btext-shadow\\b" << "\\btext-transform\\b" << "\\btop\\b" << "\\bunicode-bidi\\b" <<
            "\\bvertical-align\\b" << "\\bvisibility\\b" << "\\bvoice-family\\b" << "\\bvolume" <<
            "\\bwhite-space\\b" << "\\bwidows\\b" << "\\bwidth\\b" << "\\bword-spacing\\b" << "\\bz-index\\b" <<
            "\\bbottom\\b" << "\\bleft\\b" << "\\bheight\\b" << ";" << ":";

    foreach (QString pattern, keywordPatterns) {
        rule.pattern = QRegExp(pattern, Qt::CaseInsensitive);
        rule.format = keywordFormat;
        highlightingRules.append(rule);
    }
	/* END KEYWORDS */

	/*
	* CLASSES & IDS
	*/
    keywordFormat.setForeground(Qt::darkGreen);
    keywordFormat.setFontWeight(QFont::Bold);
    QStringList cidPatterns;

	//cidPatterns << "[#].*[{]" << "[.].*[{]" << "[}]";
	cidPatterns << ".*[{]{1}" << "[}]";

    foreach (QString pattern, cidPatterns) {
        rule.pattern = QRegExp(pattern, Qt::CaseInsensitive);
        rule.format = keywordFormat;
        highlightingRules.append(rule);
    }
	/* END CLASSES & IDS */


    classFormat.setFontWeight(QFont::Bold);
    classFormat.setForeground(Qt::darkMagenta);
    rule.pattern = QRegExp("\\bQ[A-Za-z]+\\b");
    rule.format = classFormat;
    highlightingRules.append(rule);

    //singleLineCommentFormat.setForeground(Qt::gray);
    //rule.pattern = QRegExp("//[^\n]*");
    //rule.format = singleLineCommentFormat;
    //highlightingRules.append(rule);

    multiLineCommentFormat.setForeground(Qt::gray);

    quotationFormat.setForeground(Qt::darkGreen);
    rule.pattern = QRegExp("\".*\"");
    rule.format = quotationFormat;
    highlightingRules.append(rule);

    functionFormat.setFontItalic(true);
    functionFormat.setForeground(Qt::blue);
    rule.pattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");
    rule.format = functionFormat;
    highlightingRules.append(rule);

    commentStartExpression = QRegExp("/\\*");
    commentEndExpression = QRegExp("\\*/");
}

void Highlighter::highlightBlock(const QString &text){
    foreach (HighlightingRule rule, highlightingRules) {
        QRegExp expression(rule.pattern);
        int index = text.indexOf(expression);
        while (index >= 0) {
            int length = expression.matchedLength();
            setFormat(index, length, rule.format);
            index = text.indexOf(expression, index + length);
        }
    }
    setCurrentBlockState(0);

    int startIndex = 0;
    if (previousBlockState() != 1)
        startIndex = text.indexOf(commentStartExpression);

    while (startIndex >= 0) {
        int endIndex = text.indexOf(commentEndExpression, startIndex);
        int commentLength;
        if (endIndex == -1) {
            setCurrentBlockState(1);
            commentLength = text.length() - startIndex;
        } else {
            commentLength = endIndex - startIndex
                            + commentEndExpression.matchedLength();
        }
        setFormat(startIndex, commentLength, multiLineCommentFormat);
        startIndex = text.indexOf(commentStartExpression,
                                                startIndex + commentLength);
    }
}
