#include <QtGui>
#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {
    setupFileMenu();
    setupHelpMenu();
    setupEditor();

    setCentralWidget(editor);
    setWindowTitle(tr("K-CSS editor"));
}

void MainWindow::about(){
    QMessageBox::about(this, tr("About K-CSS editor"),
                tr("<p><b>K-CSS editor</b> <br>" \
                   "Version: 0.1a <br>" \
                   "Author: cime &lt;cime3d@gmail.com&gt;</p>"));
	//highlighter = new Highlighter(editor->document());
}

void MainWindow::newFile(){
    editor->clear();
}

void MainWindow::openFile(const QString &path){
    QString fileName = path;

    if (fileName.isNull())
        fileName = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("CSS Files (*.css)"));

    if (!fileName.isEmpty()) {
        QFile file(fileName);
        if (file.open(QFile::ReadOnly | QFile::Text)){
            editor->setPlainText(file.readAll());
			setFileName(fileName);
		}
    }
}

void MainWindow::saveFile(){
	if (!currentFileName.isEmpty()){
		QApplication::setOverrideCursor(Qt::WaitCursor);
		QFile file(currentFileName);

		if (file.open(QFile::WriteOnly)){
			QTextStream out(&file);
			out << editor->toPlainText();
			file.close();
			setFileName(currentFileName);
		}
		QApplication::restoreOverrideCursor();
	} else {
		saveFileAs();
	}
}

void MainWindow::saveFileAs(){
	QString fileName;
	fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "", tr("CSS Files (*.css)"));
	QApplication::setOverrideCursor(Qt::WaitCursor);
    if (!fileName.isEmpty()) {
		//TODO: Save file
		QFile file(fileName);

		if (file.open(QFile::WriteOnly)){
			QTextStream out(&file);
			out << editor->toPlainText();
			file.close();
			setFileName(fileName);
		}
	}
	QApplication::restoreOverrideCursor();
}

void MainWindow::setFileName(const QString &file){
	setWindowTitle("K-CSS editor - " + file);
	currentFileName = file;
}

void MainWindow::setupEditor(){
    QFont font;
    font.setFamily("Arial");
    font.setFixedPitch(true);
    font.setPointSize(10);

    editor = new QTextEdit;
    editor->setFont(font);
	editor->setTabStopWidth(20);

    highlighter = new Highlighter(editor->document());
}

void MainWindow::setupFileMenu(){
    QMenu *fileMenu = new QMenu(tr("&File"), this);
    menuBar()->addMenu(fileMenu);

    fileMenu->addAction(tr("&New"), this, SLOT(newFile()), QKeySequence(tr("Ctrl+N", "File|New")));
    fileMenu->addAction(tr("&Open..."), this, SLOT(openFile()), QKeySequence(tr("Ctrl+O", "File|Open")));
    fileMenu->addAction(tr("&Save"), this, SLOT(saveFile()), QKeySequence(tr("Ctrl+S", "File|Save")));
    fileMenu->addAction(tr("&Save as..."), this, SLOT(saveFileAs()), QKeySequence(tr("Ctrl+A", "File|SaveAs")));
    fileMenu->addSeparator();
    fileMenu->addAction(tr("E&xit"), qApp, SLOT(quit()), QKeySequence(tr("Ctrl+Q", "File|Exit")));
}

void MainWindow::setupHelpMenu(){
    QMenu *helpMenu = new QMenu(tr("&Help"), this);
    menuBar()->addMenu(helpMenu);

    helpMenu->addAction(tr("&About"), this, SLOT(about()));
    helpMenu->addAction(tr("About &Qt"), qApp, SLOT(aboutQt()));
}
